﻿using System.Globalization;

using Projekt_Calcan_Conze_Export;
using Projekt_Calcan_Conze_Export.Models;
using Microsoft.VisualBasic;

try
{
    string? clientNumber = null;

    while (string.IsNullOrEmpty(clientNumber))
    {
        Console.WriteLine("Bitte gib eine Kundennummer an:");
        clientNumber = Console.ReadLine();
    }

    Console.WriteLine();

    DateOnly? startDate = null;

    while (startDate is null)
    {
        Console.WriteLine($"Bitte gib das Startdatum im folgenden Format an ({Constants.DateFormat}):");
        startDate =
            DateOnly.TryParseExact(
                Console.ReadLine() ?? string.Empty,
                Constants.DateFormat,
                CultureInfo.InvariantCulture,
                DateTimeStyles.None,
                out var start)
                ? start
                : null;
    }

    Console.WriteLine();

    DateOnly? endDate = null;

    while (endDate is null)
    {
        Console.WriteLine($"Bitte gib das Enddatum im folgenden Format an ({Constants.DateFormat}):");
        endDate =
            DateOnly.TryParseExact(
                Console.ReadLine() ?? string.Empty,
                Constants.DateFormat,
                CultureInfo.InvariantCulture,
                DateTimeStyles.None,
                out var end)
                ? end
                : null;

        if (endDate is not null
            && endDate < startDate)
        {
            Console.WriteLine("Das Enddatum darf nicht vor dem Startdatum liegen.");
            endDate = null;
        }
    }

    Console.WriteLine();

    string? directoryPath = null;

    while (string.IsNullOrEmpty(directoryPath))
    {
        Console.WriteLine("Bitte gib einen Dateipfad für die Ausgabe an:");
        directoryPath = Console.ReadLine();

        if (directoryPath?.StartsWith('\"') ?? false)
        {
            directoryPath = directoryPath.Substring(startIndex: 1, length: directoryPath.Length - 1);
        }

        if (directoryPath?.EndsWith('\"') ?? false)
        {
            directoryPath = directoryPath.Substring(startIndex: 0, length: directoryPath.Length - 1);
        }

        if (!Path.Exists(directoryPath))
        {
            Console.WriteLine("Der angegebene Dateipfad existiert nicht.");
            directoryPath = null;
        }
    }

    List<Billing> billings =
    [
        new Billing(
            startDate: startDate.Value,
            endDate: endDate.Value,
            customerNumber: clientNumber,
            customerName: "Max Mustermann",
            positions:
            [
                new BillingPosition(
                    description: "Testposition",
                    baseAmount: 100.0,
                    count: 3)
            ])
    ];
    string fileName =
        $"{DateTime.Today.ToString(Constants.DateFormat)}_{clientNumber}_{startDate.Value.ToString(Constants.DateFormat)}_{endDate.Value.ToString(Constants.DateFormat)}.csv";
    File.WriteAllLines(
        Path.Combine(directoryPath, fileName),
        billings.SelectMany(billing => billing.ToCsvLines()));
}
catch (Exception ex)
{
    Console.WriteLine($"Ein Fehler ist aufgetreten: {ex.Message}");
}